import fastifyStatic from '@fastify/static';
import fastify from 'fastify';
import { BAD_REQUEST, NO_CONTENT, badRequestSchema } from 'http-errors-enhanced';
import EventEmitter from 'node:events';
import { existsSync } from 'node:fs';
import { mkdir, readFile } from 'node:fs/promises';
import { resolve } from 'node:path';
import { Readable } from 'node:stream';
import { baseTemporaryDirectory, rootDir, serverFilePath } from './models.js';
const defaultServerOptions = {
    ip: '::',
    port: 0,
    logger: false,
    isProduction: true,
    staticDir: 'dist/html'
};
class SynchronizationEventEmitter extends EventEmitter {
    closed;
    constructor(options){
        super(options);
        this.closed = false;
        this.once('close', ()=>{
            this.closed = true;
        });
    }
    close() {
        this.emit('close');
    }
    get isClosed() {
        return this.closed;
    }
}
class SynchronizationStream extends Readable {
    emitter;
    constructor(emitter){
        super();
        emitter.on('update', ()=>{
            this.push(`event: sync\ndata: ${JSON.stringify(buildStatus)}\n\n`);
        });
        emitter.on('close', ()=>{
            this.push('event: close\ndata:\n\n');
        });
        this.emitter = emitter;
        this.push('retry: 10000\n\n');
    }
    _read() {
        if (this.emitter.isClosed) {
            this.push(null);
        }
    }
}
const buildStatus = {
    status: 'pending'
};
const buildEmitter = new SynchronizationEventEmitter();
export function notifyBuildStatus(status, payload) {
    buildStatus.status = status;
    buildStatus.payload = payload;
    buildEmitter.emit('update');
}
function syncHandler(_, reply) {
    // eslint-disable-next-line @typescript-eslint/no-floating-promises
    reply.header('content-type', 'text/event-stream').header('cache-control', 'no-cache').send(new SynchronizationStream(buildEmitter));
    setTimeout(()=>buildEmitter.emit('update'), 100);
}
export async function localServer(options) {
    const { ip, port, logger, staticDir, isProduction } = {
        ...defaultServerOptions,
        ...options
    };
    const https = existsSync(resolve(rootDir, 'ssl')) ? {
        key: await readFile(resolve(rootDir, 'ssl/privkey.pem')),
        cert: await readFile(resolve(rootDir, 'ssl/cert.pem')),
        ca: await readFile(resolve(rootDir, 'ssl/chain.pem'))
    } : null;
    let serverDir = '';
    let setupServer;
    if (existsSync(serverFilePath())) {
        const serverExport = await import(serverFilePath());
        setupServer = serverExport.setupServer;
    }
    const server = fastify({
        https,
        logger: logger ?? {
            transport: {
                target: 'pino-pretty'
            }
        },
        forceCloseConnections: true
    });
    if (setupServer && typeof setupServer === 'function') {
        const result = await setupServer(server, isProduction);
        if (result?.directory) {
            serverDir = result.directory;
        }
    }
    const root = resolve(...[
        staticDir,
        serverDir
    ].filter((s)=>s));
    await mkdir(root, {
        recursive: true
    });
    await server.register(fastifyStatic, {
        root,
        decorateReply: true,
        index: [
            'index.html',
            'index.htm'
        ]
    });
    server.decorate('rootDir', root);
    if (!isProduction) {
        server.addHook('preHandler', async (request, reply)=>{
            if (buildStatus.status !== 'success' && request.url !== '/__status') {
                return reply.sendFile('__status.html', resolve(rootDir, baseTemporaryDirectory));
            }
        });
        server.route({
            method: 'GET',
            url: '/__status',
            handler: syncHandler,
            schema: {
                response: {
                    [NO_CONTENT]: {},
                    [BAD_REQUEST]: badRequestSchema
                }
            }
        });
    }
    server.setNotFoundHandler(function(_, reply) {
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        reply.type('text/html').sendFile('404.html');
    });
    process.on('SIGINT', ()=>{
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        server.close();
    });
    return new Promise((resolve, reject)=>{
        server.listen({
            host: ip,
            port,
            listenTextResolver (address) {
                return `Server listening at ${address}.`;
            }
        }, (err)=>{
            if (err) {
                reject(err);
                return;
            }
            resolve(server);
        });
    });
}
